#!/bin/sh

MODULE_NAME="ImportExport"
MODULE_PREFIX="ImEx_"
RET=""

IMEX_HOME=/usr/iwss/$MODULE_NAME
IMEX_BIN=$IMEX_HOME/bin
IMEX_CONFIGFILE_IMPORT_BINARY=$IMEX_BIN/ImEx_ConfigFileImport
IMEX_CONFIG_INI=$IMEX_HOME/config/ImEx_Config.ini

IWSS_HOME="/var/iwss"
DEBUGLOG="0"
IMPORT_TMP_DIR="/tmp/import"
CONFIG_FILE_RULE="$IMEX_HOME/config/ImEx_ConfigFileRule.xml"
DATABASE_TABLES="tb_entity,tb_entitytype,tb_policy,tb_policytype,tb_prioritylist"

EXPORT_DIR="$IWSS_HOME/export"
CDT_EXE="$IWSS_HOME/CDT_IWSS.sh"

IMEX_LOG_PREFIX="ImEx.log"
IMEX_LOG_DIR=$IWSS_HOME/log

IMPORT_UI_LOG="/tmp/import_result.txt"
EXPORT_UI_LOG="/tmp/export_result.txt"
IMPORT_UI_POLLING_FLAG="/tmp/import_polling_progress_start"
IMEX_RES_FILE="$IWSS_HOME/resource/config_backup.txt"

PROMOTE_PERMISSION="/usr/iwss/AdminUI/uihelper"

# F_ini_get
# Get the value of a key in an INI file
# Usage: Catch the value using an variable.
# e.g. VAR=`F_ini_get "file" "section" "key"`
#
# param: full path to an INI file 
# param: section name excluding '[' ']'
# param: key name
#
# print: the ini value or error
#
F_ini_get()
{
  INI_F="$1"
  INI_SECTION="$2"
  INI_KEY="$3"
  
  if [ ! -f "$INI_F" ] ; then
    echo "Error: ${INI_F} not found."
    return 1
  fi
      
  RANGE_START="/^\[${INI_SECTION}\]/"
  RANGE_END="/^\[.*\]/"
      
  MY_SEDCMD_1="${RANGE_START},${RANGE_END}p"
  #MY_SEDCMD_2="/${INI_KEY}=/p"
  MY_SEDCMD_2="/^${INI_KEY}=/p"
  
  #_R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} | sed -n -e "${MY_SEDCMD_2}" | awk -F= '{ printf $2 }'`
  _R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} | sed -n -e "${MY_SEDCMD_2}" | cut -d'=' -f2-`
  printf "%s\n" "$_R"
  if [ -z "$_R" ] ; then
    return 1
  else
    return 0
  fi
}

# F_ini_set
# Set the value of a key in an INI file
# It uses a temp file.
# e.g. F_ini_set "file" "section" "key" "value"
#
# param: full path to an INI file 
# param: section name excluding '[' ']'
# param: key name
# param: value
#
# print: error
#
F_ini_set() {
    INI_F="$1"
    INI_SECTION="$2"
    INI_KEY="$3"
    INI_VALUE="$4"
    TMP_F=/tmp/temp$$

    if [ ! -f "$INI_F" ] ; then
        echo "Error: ${INI_F} not found."
	return 1
    fi

    RANGE_START="/^\[${INI_SECTION}\]/"
    RANGE_END="/^\[.*\]/"

    MY_SEDCMD="${RANGE_START},${RANGE_END}s?^[ \t]*${INI_KEY}=.*?${INI_KEY}=${INI_VALUE}?"
    sed -e "${MY_SEDCMD}" ${INI_F} > ${TMP_F}
    cp -f ${TMP_F} ${INI_F}
    rm -f ${TMP_F}
}

F_find_log_file() {
  today=`date "+%Y%m%d"`
  loglist=`ls $IMEX_LOG_DIR | grep "$IMEX_LOG_PREFIX.$today"`
  #echo "$loglist"
  if [ -z "$loglist" ]; then
    newest_log_file="$IMEX_LOG_PREFIX.$today.0001"
  else
    #echo "$loglist"
    newest_log_file=`echo $loglist | head -n 1 | cut -d' ' -f1`
    #echo "newest=$newest_log_file"
    for log_file in $loglist
    do
      if [ "$log_file" -nt "$newest_log_file" ]; then
        newest_log_file=$log_file
      fi
    done
  fi
  #echo "newest=$newest_log_file"
  
  printf "%s\n" "$newest_log_file"
}

F_log_initialize() {
  tz_hour=`date +%z | cut -b -3`
  tz_min=`date +%z | cut -b 4-`
  TZ_STR=`echo GMT$tz_hour:$tz_min`
  
  LOG_FILE=`F_find_log_file`
  
  _import_module=`echo "$0" | grep "Import.sh"`
  _export_module=`echo "$0" | grep "Export.sh"`
  if [ -n "$_import_module" ]; then
    MODULE="Import"
    rm -rf $IMPORT_UI_LOG
    rm -rf $IMPORT_UI_POLLING_FLAG
  fi
  if [ -n "$_export_module" ]; then
    MODULE="Export"
    rm -rf $EXPORT_UI_LOG
  fi
}

F_debug_log() {
  if [ "$DEBUGLOG" = "1" ]; then
    echo "$1"
    if [ "$LOG_FILE" != "" ]; then
      echo "`date "+%Y/%m/%d %H:%M:%S"` $TZ_STR [$MODULE] $1" >> $IMEX_LOG_DIR/$LOG_FILE
    fi
  fi
}

F_log() {
  echo "$1"
  if [ "$LOG_FILE" != "" ]; then
    echo "`date "+%Y/%m/%d %H:%M:%S"` $TZ_STR [$MODULE] $1" >> $IMEX_LOG_DIR/$LOG_FILE
  fi
}

F_UI_log() {
  _res_file=$1
  _res_sec=$2
  _res_key=$3
  _prefix=$4
  _extra_str=$5
  
  if [ ! -f "$_res_file" ]; then
    return 1
  fi  
  _res_value=`F_ini_get $_res_file $_res_sec $_res_key`
  
  if [ -n "$_extra_str" ]; then
    _res_value=$_res_value$_extra_str
  fi
  
  if [ "$DEBUGLOG" = "1" ]; then
    echo "[UI_LOG] [[$_prefix]] $_res_value"
  fi
  if [ "$MODULE" = "Import" ]; then
    echo "[[$_prefix]] $_res_value" >> $IMPORT_UI_LOG
  elif [ "$MODULE" = "Export" ]; then
    echo "[[$_prefix]] $_res_value" >> $EXPORT_UI_LOG
  fi
  return 0
}

F_check_user() {
  _user=$1
  _group=$2
  _cur_user=`id -un`
  _cur_group=`id -gn`
  if [ "$_cur_user" != "$_user" ]; then
    return 1
  fi
  if [ "$_cur_group" != "$_group" ]; then
    return 1
  fi
  return 0
}

F_UI_polling_start() {
	# touch a file to notify UI start to polling.
  touch $IMPORT_UI_POLLING_FLAG
}

###
F_IsMultipleProcess() {
  
  RET="true"
  
  # First Check for Multiple Import/Export Running
  if [ -f "$IWSS_HOME/$MODULE_NAME.pid" ] ; then
    _previous_pid=`cat $IWSS_HOME/$MODULE_NAME.pid`
  fi
  	
  if [ -z "$_previous_pid" ] ; then
    # No other Import/Export is running
    RET="false"
  else
    _previous_pidinfo=`ps aux | grep "$_previous_pid" | grep -v 'grep' | grep -v '\<defunct\>' | grep "$MODULE_PREFIX"`
    if [ -z "$_previous_pidinfo" ] ; then  
  		# Second Check for Import/Export Running ####
  		if [ -f "$IWSS_HOME/$MODULE_NAME.pid" ] ; then
        _previous_pid=`cat $IWSS_HOME/$MODULE_NAME.pid`
  		fi
  		if [ -z "$_previous_pid" ] ; then
        # No other Import/Export is running
        RET="false"
  		else
  			_previous_pidinfo=`ps aux | grep "$_previous_pid" | grep -v 'grep' | grep -v '\<defunct\>' | grep "$MODULE_PREFIX"`
  			
  			if [ -z "$_previous_pidinfo" ] ; then		
  				#No other Import/Export is running
  				RET="false"
  			fi
  		fi
    fi
  fi
  	
  printf "%s\n" "$RET"
  return 0
}

###
F_lock() {
  _current_pid=$$
  echo $_current_pid > $IWSS_HOME/$MODULE_NAME.pid
  chmod a+rw $IWSS_HOME/$MODULE_NAME.pid > /dev/null 2> /dev/null
  chown iscan:iscan $IWSS_HOME/$MODULE_NAME.pid
}
