#!/bin/sh

. ./util_source.sh
. ./arm_activation_common.sh

export PATH=$PG_BIN:$PATH

PSQL_CMD=/usr/iwss/PostgreSQL/bin/psql
TMP_DIR=/etc/iscan/tmp

tar_file=$1; shift
tables=$1; shift
sequences=$1; shift

clearCsvFiles()
{
	csvFiles=*.csv
	if [ "*.csv" != "$csvFiles" ] ; then
		rm *.csv
	fi
}

cleanup()
{
	clearCsvFiles
	if [ "" != "$cmd_file" ] ; then
		rm $cmd_file 2> /dev/null
	fi
}

error_exit()
{
	cleanup
	exit 1
}

handleOperationErr()
{
	trap 10
	echo "Script execution aborted" 1>&2
	error_exit
}

trap "handleOperationErr" 10

handleEnvErr()
{
	trap 1 2 3 4 5 6 7 8 11 12 13 14 15
	echo "Script execution terminated: execution problem" 1>&2
	error_exit
}

trap "handleEnvErr" 1 2 3 4 5 6 7 8 11 12 13 14 15

#
# migrateImport.sh Tables.tar 10.2.42.238 "tb_policy tb_rule tb_entity tb_prioritylist tb_entitytype tb_policytype" "tb_policy_PolicyID_seq tb_rule_RuleID_seq tb_entity_EntityID_seq"
#

getOdbcVal()
{
	xval=`util_getIniValue $ODBCINI_FILE = $1 $2`
	if [ "" = "$xval" ] ; then
		util_funcErr getOdbcVal "failed to get [$1]$2 from $ODBCINI_FILE"
		return 1
	fi
	echo $xval
}

db_hostname=`getOdbcVal IWSS Servername`;
if [ 0 != $? ] ; then
	error_exit
fi
db_port=`getOdbcVal IWSS Port`;
if [ 0 != $? ] ; then
	error_exit
fi
db_user=`getOdbcVal IWSS UserName`;
if [ 0 != $? ] ; then
	error_exit
fi
db_name=`getOdbcVal IWSS Database`;
if [ 0 != $? ] ; then
	error_exit
fi
db_pw=`getOdbcVal IWSS Password`;
if [ 0 != $? ] ; then
	error_exit
fi


#tables="tb_policy tb_rule tb_entity tb_prioritylist tb_entitytype tb_policytype"
#sequences="tb_policy_PolicyID_seq tb_rule_RuleID_seq tb_entity_EntityID_seq"

extractTar()
{
	tar xzf $tar_file 2>/dev/null
	if [ 0 -ne $? ] ; then
		util_funcErr extractTar "failed to untar import package: $tar_file"
		error_exit
	fi
}

getSchemaName()
{
	xstuff=`$PSQL_CMD -h $db_hostname -p $db_port -U $db_user -d $db_name -c "\d tb_policy;" | grep -i "from.*farm_schema_.*tb_policy" | sed -e "s/.*FROM \(farm_schema_.*\).tb_policy.*/\1/" 2>/dev/null`
	if [ "" != "$xstuff" ] ; then
		xstuff="$xstuff."
	fi
	#xstuff=`$PSQL_CMD -h $db_hostname -p $db_port -U $db_user -d $db_name -c "\d tb_policy"  | grep "FROM farm_schema_.*\.tb_policy" -o`
	#echo "SCHEMA QUERY RESULT: $xstuff" 1>&2
	#xstuff=`echo $xstuff | grep "farm_schema_.*\." -o`
	echo "$xstuff"
}

getColumnFromSeq()
{
	echo `echo $1 | cut -d_ -f3`
}

getTblFromSeq()
{
	base=`echo $1 | cut -d_ -f2`
	echo "tb_$base"
}

initCmdFile()
{
	cmdFile="cmd_$$.txt"
	echo "\set ON_ERROR_STOP" > $cmdFile
	echo "BEGIN;" >> $cmdFile
	echo $cmdFile
}

finishCmdFile()
{
	echo "COMMIT;" >> $cmd_file
}

genLocks()
{
	for tbl
	do
		echo "LOCK TABLE ${db_schema}$tbl;" >> $cmd_file
	done
}

genDeletes()
{
	for tbl
	do
		echo "DELETE FROM ${db_schema}$tbl;" >> $cmd_file
	done
}

genCopies()
{
	# -- /bin/echo "\copy" - the "\c" can cause problems, so emit the '\' separately
	for tbl
	do
		echo -n '\' >> $cmd_file
		echo "copy ${db_schema}$tbl from $tbl.csv with delimiter ','" >> $cmd_file
	done
}

genSeqResets()
{
	for seq
	do
		idcol=`getColumnFromSeq $seq`
		tbl=`getTblFromSeq $seq`
		# -- Postgres 8.3
		#echo "SELECT setval('${db_schema}\"$seq\"', GREATEST((SELECT MAX(\"$idcol\") FROM ${db_schema}$tbl), (nextval('${db_schema}\"$seq\"') - 1)));" >> $cmd_file
		# -- Postgres 7.4 - no GREATEST function
		echo "SELECT setval('${db_schema}\"$seq\"', 
                            CASE
	                            WHEN (SELECT MAX(\"$idcol\") FROM ${db_schema}$tbl) > (nextval('${db_schema}\"$seq\"') - 1) THEN (SELECT MAX(\"$idcol\") FROM ${db_schema}$tbl)
	                            ELSE (currval('${db_schema}\"$seq\"') - 1)
                            END
                            );" >> $cmd_file
	done
}

loadTables()
{
	$PSQL_CMD -h $db_hostname -p $db_port -U $db_user -d $db_name -f $cmd_file
	rc=$?
	if [ 0 -ne $rc ] ; then
		util_funcErr loadTables "Load of tables via PSQL failed: rc = $rc"
		error_exit
	fi
	echo "TABLES LOADED: RC = $rc"
}

work_dir=$TMP_DIR/armmigrate_$$
mkdir $work_dir
if [ ! -x $work_dir ] ; then
	util_funcErr main "failed to create work directory: $work_dir"
	exit 1
fi

cd $work_dir
trap "cd /etc/iscan; rm -rf $work_dir" 0

extractTar

db_schema=`getSchemaName`
echo "SCHEMA NAME IS: [$db_schema]"

cmd_file=`initCmdFile`

genLocks $tables
genDeletes $tables
genCopies $tables
genSeqResets $sequences

finishCmdFile

loadTables

cleanup

exit 0
