#!/bin/sh

# ------------------------------------------------------------------------------
# -- ERROR HANDLING/REPORTING

util_die()
{
	kill -USR1 $$
}

util_funcFatal()
{
	echo "ERROR: function $1: $2" 1>&2
	util_die
}

util_funcErr()
{
	echo "ERROR: function $1: $2" 1>&2
}

util_funcWarn()
{
	echo "WARNING: function $1: $2" 1>&2
}

# ------------------------------------------------------------------------------
# -- FUNCTION PARAMETER VALIDATION

util_funcParamErr()
{
	util_funcErr $1 "parameter $2: $3"
}

util_paramHasValue()
{
	if [ -z "$3" ] ; then
		util_funcParamErr $1 $2 "parameter not specified"
		return 1
	fi
	return 0
}

util_readableFileParam()
{
	util_paramHasValue $1 $2 "$3"
	if [ 0 -ne $? ] ; then
		return 1
	fi
	if [ ! -r "$3" ] ; then
		util_funcParamErr $1 $2 "file is not readable or does not exist: $3"
		return 1
	fi
	return 0
}

util_writeableFileParam()
{
	util_paramHasValue $1 $2 "$3"
	if [ 0 -ne $? ] ; then
		return 1
	fi
	if [ ! -r "$3" ] ; then
		# file does not exist
		return 0
	fi
	if [ ! -f "$3" ] ; then
		util_funcParamErr $1 $2 "file exists but is not a regular file: $3"
		return 1
	fi
	if [ ! -w "$3" ] ; then
		util_funcParamErr $1 $2 "file exists but is not writeable: $3"
		return 1
	fi
	return 0
}

util_writeableDirParam()
{
	util_paramHasValue $1 $2 "$3"
	if [ 0 -ne $? ] ; then
		return 1
	fi
	if [ ! -r "$3" ] ; then
		util_funcParamErr $1 $2 "directory does not exist: $3"
		return 1
	fi
	if [ ! -d "$3" ] ; then
		util_funcParamErr $1 $2 "file exists but is not a directory: $3"
		return 1
	fi
	if [ ! -w "$3" ] ; then
		util_funcParamErr $1 $2 "directory exists but is not writeable: $3"
		return 1
	fi
	return 0
}

# ------------------------------------------------------------------------------
# -- MISC FUNCTIONS

util_rmFileEx()
{
	util_rmFile "$1"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_rmFile()
{
	if [ ! -r "$1" ] ; then
		return 0
	fi
	rm -f "$1"
	if [ $? -ne 0 ] ; then
		util_funcErr util_rmFile "Failed to remove file: $1"
		return 1
	fi
	return 0
}

util_mvFileEx()
{
	util_mvFile "$1" "$2"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_mvFile()
{
	mv -f "$1" "$2"
	if [ $? -ne 0 ] ; then
		util_funcErr util_mvFile "Failed to move file $1 to $2"
		return 1
	fi
	return 0
}

util_cpFileEx()
{
	util_cpFile "$1" "$2"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_cpFile()
{
	cp -pf "$1" "$2"
	if [ $? -ne 0 ] ; then
		util_funcErr util_cpFile "Failed to copy file $1 to $2"
		return 1
	fi
	return 0
}

# ------------------------------------------------------------------------------
# -- SAVEFILE FUNCTIONS

saveDir=

saveFile_initEx()
{
	saveFile_init $1
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

saveFile_init()
{
	saveDir=$1
	if [ ! -r $saveDir ] ; then
		mkdir -p $saveDir
	fi
	
	if [ ! -d  $saveDir ] ; then
		util_funcErr saveFile_init "Failed to create saved file directory $saveDir"
		return 1
	fi
	if [ ! -w  $saveDir ] ; then
		util_funcErr saveFile_init "Saved file directory is not writeable: $saveDir"
		return 1
	fi
	return 0
}

saveFile_saveEx()
{
	saveFile_save "$1"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

saveFile_savedFile()
{
	if [ -z $saveDir ] ; then
		util_funcErr saveFile_save "Not initialized"
		return 1
	fi
	
	fname=${1##*/}
	if [ ! -r "$saveDir/$fname" ] ; then
		util_funcErr saveFile_restore "Saved file not found in $saveDir: $fname"
		return 1
	fi
	
	echo $saveDir/$fname
}

saveFile_save()
{
	if [ -z $saveDir ] ; then
		util_funcErr saveFile_save "Not initialized"
		return 1
	fi
	
	if [ ! -r $1 ] ; then
		util_funcErr saveFile_save "File is not readable: $1"
		return 1
	fi

	fname=${1##*/}
	cp -pf $1 $saveDir/$fname
	if [ $? -ne 0 ] ; then
		util_funcErr saveFile_save "Failed to save file: $1"
		return 1
	fi
	
	return 0
}

saveFile_restoreEx()
{
	saveFile_restore "$1"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

saveFile_restore()
{
	if [ -z $saveDir ] ; then
		util_funcErr saveFile_restore "Not initialized"
		return 1
	fi
	
	destDir=${1%/*}
	fname=${1##*/}
	
	if [ ! -r "$saveDir/$fname" ] ; then
		util_funcErr saveFile_restore "Saved file not found in $saveDir: $fname"
		return 1
	fi
	
	echo "Restoring $fname to $destDir"

	cp -pf $saveDir/$fname $1
	if [ $? -ne 0 ] ; then
		util_funcErr saveFile_restore "Failed to restore file: $1"
		return 1
	fi
	return 0
}

# ------------------------------------------------------------------------------
# -- BKUP FUNCTIONS

bkupList=
bkupDir=

bkup_initEx()
{
	bkup_init $1
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

bkup_init()
{
	bkupDir=$1
	mkdir -p $bkupDir
	
	if [ ! -d  $bkupDir ] ; then
		util_funcErr bkup_init "Failed to create backup directory $bkupDir"
		return 1
	fi
	if [ ! -w  $bkupDir ] ; then
		util_funcErr bkup_init "Backup directory is not writeable: $bkupDir"
		return 1
	fi
	return 0
}

bkup_cleanup()
{
	rm -rf $bkupDir
}

bkup_restoreFile()
{
	if [ -z $bkupDir ] ; then
		util_funcErr bkup_restoreFile "Not initialized"
		return 1
	fi
	
	destDir=${1%/*}
	fname=${1##*/}
	
	if [ ! -r "$bkupDir/$fname" ] ; then
		util_funcErr bkup_restoreFile "Backup file not found in $bkupDir: $fname"
		return 1
	fi
	
	echo "Restoring $fname to $destDir"

	cp -pf $bkupDir/$fname $1
	if [ $? -ne 0 ] ; then
		util_funcErr bkup_restoreFile "Failed to restore file: $1"
		return 1
	fi
	return 0
}

bkup_pushFileEx()
{
	bkup_pushFile "$1"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

bkup_pushFile()
{
	if [ -z $bkupDir ] ; then
		util_funcErr bkup_pushFile "Not initialized"
		return 1
	fi
	
	if [ ! -r $1 ] ; then
		util_funcErr bkup_pushFile "File is not readable: $1"
		return 1
	fi

	fname=${1##*/}
	cp -pf $1 $bkupDir/$fname
	if [ $? -ne 0 ] ; then
		util_funcErr bkup_pushFile "Failed to backup file: $1"
		return 1
	fi

	bkupList="$1 $bkupList"
	
	return 0
}

bkup_restoreFiles()
{
	if [ -z $bkupDir ] ; then
		return 1
	fi
	
	for fl in $bkupList ; do
		bkup_restoreFile $fl
	done
	bkupList=
	return 0
}

# ------------------------------------------------------------------------------
# -- INI FILE MANIPULATION FUNCTIONS

util_changeIniValueInPlaceEx()
{
	util_changeIniValueInPlace "$1" "$2" "$3" "$4" "$5" "$6"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_changeIniValueInPlace()
{
	srcFile=$1
	tmpFile=$2
	util_changeIniValue "$srcFile" "$tmpFile" "$3" "$4" "$5" "$6"
	if [ $? -ne 0 ] ; then
		return 1
	fi
	util_mvFile $tmpFile $srcFile
	return $?
}

util_changeIniValueEx()
{
	util_changeIniValue "$1" "$2" "$3" "$4" "$5" "$6"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_changeIniValue()
{
	srcFile=$1
	destFile=$2
	sep=$3
	section=$4
	key=$5
	value=$6

	func=util_changeIniValue
	util_readableFileParam $func srcFile "$srcFile" || return 1;
	util_writeableFileParam $func destFile "$destFile" || return 1;
	util_paramHasValue $func sep "$sep" || return 1;
	util_paramHasValue $func section "$section" || return 1;
	util_paramHasValue $func key "$key" || return 1;
	util_paramHasValue $func value "$value" || return 1;
	
	hasValue=`util_hasIniValue "$srcFile" "$sep" "$section" "$key"`
	if [ "1" != "$hasValue" ] ; then
		util_funcErr $func "[$section]$key does not exist in $srcFile"
		return 1
	fi

	awk -f setIniVal.awk -v section="$section" -v sep="$sep" -v key="$key" -v value="$value" $srcFile > $destFile
	
	# -- remove leading and trailing whitespace for check only
	util_checkVal "$destFile" "$sep" "$section" "$key" $value
	if [ $? -ne 0 ] ; then
		util_funcErr $func "Value for [$section]$key not set correctly in $destFile"
		return 1
	fi

	return 0
}

util_checkVal()
{
	fl=$1
	shift
	sep=$1
	shift
	section=$1
	shift
	key=$1
	shift
	val=$*

	util_paramHasValue util_checkVal value "$val" || return 1;

	checkValue=`util_getIniValue "$fl" "$sep" "$section" "$key"`
	if [ "$checkValue" != "$val" ] ; then
		return 1
	fi

	return 0
}

util_setIniValueInPlaceEx()
{
	util_setIniValueInPlace "$1" "$2" "$3" "$4" "$5" "$6"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_setIniValueInPlace()
{
	srcFile=$1
	tmpFile=$2
	util_setIniValue "$srcFile" "$tmpFile" "$3" "$4" "$5" "$6"
	if [ $? -ne 0 ] ; then
		return 1
	fi
	util_mvFile $tmpFile $srcFile
	return $?
}

util_setIniValueEx()
{
	util_setIniValue "$1" "$2" "$3" "$4" "$5" "$6"
	if [ $? -ne 0 ] ; then
		util_die
	fi
}

util_setIniValue()
{
	srcFile=$1
	destFile=$2
	sep=$3
	section=$4
	key=$5
	value=$6
	
	func=util_setIniValue
	util_readableFileParam $func srcFile "$srcFile" || return 1;
	util_writeableFileParam $func destFile "$destFile" || return 1;
	util_paramHasValue $func sep "$sep" || return 1;
	util_paramHasValue $func section "$section" || return 1;
	util_paramHasValue $func key "$key" || return 1;
	util_paramHasValue $func value "$value" || return 1;

	awk -f setIniVal.awk -v section="$section" -v sep="$sep" -v key="$key" -v value="$value" $srcFile > $destFile
	
	# -- remove leading and trailing whitespace from value for check only
	util_checkVal "$destFile" "$sep" "$section" "$key" $value
	if [ $? -ne 0 ] ; then
		util_funcErr $func "Value for [$section]$key not set correctly in $destFile"
		return 1;
	fi

	return 0;
}

util_getIniValue()
{
	fl=$1
	sep=$2
	section=$3
	key=$4
	
	func=util_getIniValue
	util_readableFileParam $func fl "$fl" || return 1;
	util_paramHasValue $func sep "$sep" || return 1;
	util_paramHasValue $func section "$section" || return 1;
	util_paramHasValue $func key "$key" || return 1;

	val=`awk -f getIniVal.awk -v section="$section" -v sep="$sep" -v key="$key" $fl`
	# -- remove leading and trailing whitespace
	val=$val
	echo -n $val
}

util_hasIniValue()
{
	fl=$1
	sep=$2
	section=$3
	key=$4
	
	func=util_hasIniValue
	util_readableFileParam $func fl "$fl" || return 1;
	util_paramHasValue $func sep "$sep" || return 1;
	util_paramHasValue $func section "$section" || return 1;
	util_paramHasValue $func key "$key" || return 1;

	awk -f hasIniVal.awk -v section="$section" -v sep="$sep" -v key="$key" $fl
}

util_hasIniSection()
{
	fl=$1
	section=$2
	
	func=util_hasIniValue
	util_readableFileParam $func fl "$fl" || return 1;
	util_paramHasValue $func section "$section" || return 1;

	awk -f hasIniSection.awk -v section="$section" $fl
}
